#include <GL/glew.h>

#include <iostream>
#include <fstream>
#include <sstream>
#include <string.h>

typedef ptrdiff_t ourGLsizeiptr;
typedef char ourGLchar;


// Variables globales
GLuint vertexShader;
GLuint fragmentShader;

void checkError(const char *functionName);

/** Charge le fichier texte dont le nom est pass en paramtre */
std::string LoadSource(const std::string & fileName)
{
	std::ifstream fichier(fileName.c_str());

	if(fichier)
	{
		std::stringstream buffer;
		buffer << fichier.rdbuf();
		fichier.close();
		std::string source(buffer.str());
		source.push_back('\0');
		return source;
	}
	return "";
}

/** Vrifie l'tat d'un shader */
void checkProgramInfos(GLuint program, GLuint stat){
	GLint status;
	glGetProgramiv(program, stat, &status);
	if(status == GL_FALSE) {
		GLint infoLogLength;
		ourGLchar *infoLog;
		glGetProgramiv(program, GL_INFO_LOG_LENGTH, &infoLogLength);
		infoLog =(ourGLchar*) malloc(infoLogLength);
		glGetProgramInfoLog(program, infoLogLength, NULL, infoLog);
		fprintf(stderr, "link log: %s\n", infoLog);
		free(infoLog);
	}
}

/** Compilation et linkage des shaders GLSL */

void compileAndCheck(GLuint shader)
{

	//Compilation
	glCompileShader(shader);

	//Vrification des possibles erreurs de compilation
	GLint status;
	glGetShaderiv (shader, GL_COMPILE_STATUS, &status);
	if (status == GL_FALSE) {
     GLint infoLogLength;
     ourGLchar *infoLog;
     glGetShaderiv (shader, GL_INFO_LOG_LENGTH, &infoLogLength);
     infoLog = (ourGLchar*) malloc (infoLogLength);
     glGetShaderInfoLog (shader, infoLogLength, NULL, infoLog);
     fprintf (stderr, "compile log: %s\n", infoLog);
     free (infoLog);
   }
}

GLuint compileShaderSource(GLenum type, GLsizei count, const ourGLchar **string, GLuint shaderID)
{
	if(shaderID==0){
		//Creation du shader
		shaderID = glCreateShader(type);
	}

	//Chargement des sources
	glShaderSource(shaderID, count, string, NULL);
	//Compilation du shader
	compileAndCheck(shaderID);

	return shaderID;
}

void linkAndCheck(GLuint program)
{
	// Linkage du programme
	glLinkProgram(program);

	// Vrification des possibles erreurs de linkage
	checkProgramInfos(program, GL_LINK_STATUS);

	glValidateProgram(program);
	checkProgramInfos(program, GL_VALIDATE_STATUS);
}

GLuint createShaderProgram(std::string &fileNameVS, std::string &fileNameFS, GLuint program=0)
{
	bool reload=program!=0;

	// Chargement des sources depuis les fichiers texte
	std::string vertexShaderString = LoadSource(fileNameVS);
	std::string fragmentShaderString = LoadSource(fileNameFS);
	const ourGLchar * vertexShaderSource = vertexShaderString.c_str();
	const ourGLchar * fragmentShaderSource = fragmentShaderString.c_str();

	const GLsizei vertexShaderLines = sizeof(vertexShaderSource) / sizeof(ourGLchar*);
	
	if(!reload)
		vertexShader = compileShaderSource(GL_VERTEX_SHADER, vertexShaderLines, &vertexShaderSource, 0);
	else
		compileShaderSource(GL_VERTEX_SHADER, vertexShaderLines, &vertexShaderSource, vertexShader);

	checkError("createVertexShader");

	const GLsizei fragmentShaderLines = sizeof(fragmentShaderSource) / sizeof(ourGLchar*);
	
	if(!reload)
		fragmentShader = compileShaderSource(GL_FRAGMENT_SHADER, fragmentShaderLines, &fragmentShaderSource, 0);
	else
		compileShaderSource(GL_FRAGMENT_SHADER, fragmentShaderLines, &fragmentShaderSource, fragmentShader);

	checkError("createFragmentShader");

	// Cration du programme
	if(!reload){
		program = glCreateProgram();
	}

	// Ajout du vertex shader
	if(vertexShader != 0) {
		glAttachShader(program, vertexShader);
	}
	// Ajout du fragment shader
	if(fragmentShader != 0) {
		glAttachShader(program, fragmentShader);
	}
	// Linkage
	linkAndCheck(program);

	return program;
}