#include "Sphere.h"
#include <math.h>

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

/** 
* Sphere constructor
* 
*/
Sphere::Sphere(float radius, int nPhi, int nTheta)
{
	float phiInc   = M_PI/(nPhi-1.);
	float thetaInc = 2.*M_PI/(nTheta-1.);

	float phi, theta;
	int i, j;

	for (j=0; j<nPhi; j++) {

		phi = -M_PI/2. + phiInc * j; 

		for (i=0; i<nTheta; i++) {

			LVector4 vertex;
			LVector3 tangent, normale;
			LVector2 texCoord;

			theta = thetaInc * i;

			// position
			vertex.x = radius*cos(theta)*cos(phi);
			vertex.y = radius*sin(theta)*cos(phi);
			vertex.z = radius*sin(phi);
			vertex.w = 1.0;

			// normal
			normale.x = cos(theta) * cos(phi);
			normale.y = sin(theta) * cos(phi);
			normale.z = sin(phi);

			// tangent
			tangent.x = -vertex.y;
			tangent.y = vertex.x;
			tangent.z = 0;

			// texture coordinates
			texCoord.x = 1.0-i/float(nTheta-1.);
			texCoord.y = j/float(nPhi-1.);

			_vertices.push_back  (vertex);
			_normalsV.push_back  (normale);
			_tangentsV.push_back (tangent);
			_texCoordsV.push_back(texCoord);
		}    
	}

	for (j=0; j<nPhi-1; j++) {
		for (i=0; i<nTheta-1; i++) {

			LTriangle tri1, tri2;

			tri1.a=(j*nTheta + i);
			tri1.b=(((j+1)%nPhi)*nTheta + i);
			tri1.c=(((j+1)%nPhi)*nTheta + (i+1)%nTheta);

			tri2.a=(j*nTheta + i);
			tri2.b=(((j+1)%nPhi)*nTheta + (i+1)%nTheta);
			tri2.c=(j*nTheta + (i+1)%nTheta);

			_triangles.push_back(tri1);
			_triangles.push_back(tri2);      
		}
	}
}

unsigned short *Sphere::getModelIndices(){
	return (unsigned short *)&_triangles[0];
}

int Sphere::getModelNumIndices(){
	return _triangles.size()*3;
}

GLfloat *Sphere::getModelVertices(){
	return (GLfloat *)&_vertices[0];
}

GLfloat *Sphere::getModelUV() {
	return (GLfloat*)&_texCoordsV[0];
}

GLfloat *Sphere::getModelNormals() {
	return (GLfloat*)&_normalsV[0];
}

GLfloat *Sphere::getModelTangents() {
	return (GLfloat*)&_tangentsV[0];
}

void Sphere::initAttrib(GLuint shaderProgramID){

	//Vertices positions
	//Get attribute ID
	GLuint attribID=glGetAttribLocation( shaderProgramID, "VertexPosition" );
	if(attribID != -1){
		//Connect data array to attribute
		glVertexAttribPointer( attribID, 4, GL_FLOAT, GL_FALSE, 0, getModelVertices());
		//Attribute array will be used for rendering
		glEnableVertexAttribArray( attribID );
	}

	//Vertices normals
	attribID=glGetAttribLocation( shaderProgramID, "VertexNormal" );
	if(attribID != -1){
		glVertexAttribPointer( attribID, 3, GL_FLOAT, GL_FALSE, 0, getModelNormals());
		glEnableVertexAttribArray( attribID );
	}

	//Vertices UV
	attribID=glGetAttribLocation( shaderProgramID, "VertexUV" );
	if(attribID != -1){
		glVertexAttribPointer( attribID, 2, GL_FLOAT, GL_FALSE, 0, getModelUV());
		glEnableVertexAttribArray( attribID );
	}

	//Vertices Tangents
	attribID=glGetAttribLocation( shaderProgramID, "VertexTangent" );
	if(attribID != -1){
		glVertexAttribPointer( attribID, 3, GL_FLOAT, GL_FALSE, 0, getModelTangents());
		glEnableVertexAttribArray( attribID );
	}

	//Vertices Binormal
	/*attribID=glGetAttribLocation( shaderProgramID, "VertexBinormal" );
	if(attribID != -1){
		glVertexAttribPointer( attribID, 3, GL_FLOAT, GL_FALSE, 0, getModelBinormals());
		glEnableVertexAttribArray( attribID );
	}*/
}