// GLEW header pour OpenGL
#include <GL/glew.h>
#include <GL/wglew.h>
// FreeGlut pour grer le fentrage
#include <GL/freeglut.h>
// OpenGL Mathematics pour grer les vecteurs/matrices
#include <glm/glm.hpp>
#include <glm/gtc/matrix_projection.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <glm/gtx/transform.hpp>
#include <glm/gtx/matrix_operation.hpp>
#include <glm/gtx/type_ptr.hpp>
using namespace glm;

// Chargement de maillages et textures
#include "Helpers.h"

// Pour crer un sphre
#include "Sphere.h"


/** Variables globales **/

// dimensions initiales de la fentre
float height = 500.0f;
float width = 500.0f;

// Vue de la camra
CView	MyView;
// Vue de la lampe
CView	Light;
// Souris
CMouse	MyMouse;

// Matrice de projection
mat4 projectionMatrix;
// Matrice Modle -> Monde
mat4 modelWorldMatrix;
// Matrice Monde -> Vue
mat4 worldViewMatrix;

// Les trois premires valeurs de chaque ligne sont les couleurs 
// et les trois suivantes les coordonnes
GLfloat CubeArray[48] = {
	1.0f, 0.0f, 0.0f, -1.0f, 1.0f, -1.0f,  
	1.0f, 0.0f, 1.0f, -1.0f, -1.0f, -1.0f,  
	1.0f, 1.0f, 1.0f, -1.0f, 1.0f, 1.0f,
	0.0f, 0.0f, 1.0f, -1.0f, -1.0f, 1.0f,   
	1.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f,     
	1.0f, 1.0f, 0.0f, 1.0f, -1.0f, 1.0f,   
	1.0f, 0.0f, 1.0f, 1.0f, 1.0f, -1.0f,   
	1.0f, 1.0f, 0.0f, 1.0f, -1.0f, -1.0f   
};

// Les indices des vertices
GLuint IndiceArray[36] = {
	0,1,2,2,1,3,
	4,5,6,6,5,7,
	3,1,5,5,1,7,
	0,2,6,6,2,4,
	6,7,0,0,7,1,
	2,3,4,4,3,5
};

// Une sphre
Sphere _sphere;

enum {
  numColorComponents = 3,
  numVertexComponents = 3,
  stride = sizeof(GLfloat) * (numColorComponents + numVertexComponents),
};


// Dessin d'un quad :
// 2 premires valeurs = coordonnes de texture
// 2 suivantes = positions
GLfloat quadArray[16] = {
	0.0f, 0.0f, -1.f, 1.f,
	1.0f, 0.0f, 1.f, 1.f,
	1.0f, 1.0f, 1.f, -1.f,
	0.0f, 1.0f, -1.f, -1.f
};;
// Les indices des vertices du quad
GLuint quadIndiceArray[4] = {0,1,2,3};

// Nom des vertex buffers object
GLuint cubeBuffers[2];
GLuint quadBuffer[2];

// Indice des vertex array objects
GLuint vertexArrayObject;
GLuint quadVAO;

// Indices des textures de couleur et de bump
GLuint id_tex, id_bump_tex;

// Indice du framebuffer object
GLuint fbo;
// Indice de la texture de la carte de profondeur
GLuint depthMapTex;
// Indice de la texture de la carte de couleur
GLuint colorMapTex;

// Identifiant des shaders
GLuint program;
GLuint quadProgram;

// Indices dans les shaders des variable uniformes et attributs
GLuint ProjectionMatrixIndex;
GLuint ModelViewMatrixIndex;
GLuint VertexColorIndex;
GLuint VertexPositionIndex;
GLuint LightPosIndex;
GLuint ColorTexIndex;
GLuint DepthTexIndex;
GLuint ThresholdIndex;
GLuint WidthIndex;
GLuint HeightIndex;

// Position de la lampe
vec3 lightPos(2.0, 2.0, -2.0);

// dessin du cube : 0 - du maillage 3ds : 1 - deffered shading : 2
int shading;

/** Cration d'un shader (dfini dans ShadersManagemnt.cpp) */
GLuint createShaderProgram(std::string &fileNameVS, std::string &fileNameFS, GLuint programID=0);


/*************************************************/

/** Cration et initialisation des buffers */
void initBuffer(void)
{
	// Gnration des buffers
	glGenBuffers(2, cubeBuffers);
	
	// Buffer d'informations de vertices
	glBindBuffer(GL_ARRAY_BUFFER, cubeBuffers[0]);
	glBufferData(GL_ARRAY_BUFFER, sizeof(CubeArray), CubeArray, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	
	// Buffer d'indices
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, cubeBuffers[1]);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(IndiceArray), IndiceArray, GL_STATIC_DRAW);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
   
	checkError ("initBuffer");
}

/** Cration et initialisation du framebuffer object et de la carte de profondeur */
void initFBO(void)
{
	// Cration du framebuffer object
	glGenFramebuffers(1,&fbo);
	
	// Activer le framebuffer object en lecture / criture
	glBindFramebuffer(GL_FRAMEBUFFER, fbo);

	// Cration d'une texture pour stocker la profondeur
	glGenTextures(1, &depthMapTex);
	glBindTexture(GL_TEXTURE_2D, depthMapTex);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, width, height, 0, GL_DEPTH_COMPONENT, GL_FLOAT, NULL);

	// Cration d'une texture pour stocker la couleur
	glGenTextures(1, &colorMapTex);
	glBindTexture(GL_TEXTURE_2D, colorMapTex);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
	
	// Attacher ces texture au framebuffer
	glFramebufferTexture2D(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_TEXTURE_2D, depthMapTex, 0);
	glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, colorMapTex, 0);

	// Desactiver le fbo
	glBindFramebuffer(GL_FRAMEBUFFER, 0);

	checkError ("initFBO");
}


/** Initialisation des shaders */
void initShader(void)
{
	program = createShaderProgram(std::string("Shaders/smoothColor.vp"),std::string("Shaders/smoothColor.fp"), 0);

	// Rcupration de l'emplacement des variables uniformes dans le shader
	ProjectionMatrixIndex = glGetUniformLocation(program, "ProjectionMatrix");
	LightPosIndex = glGetUniformLocation(program, "LightPos");
	ModelViewMatrixIndex = glGetUniformLocation(program, "ModelViewMatrix");

	// Rcupration des emplacements des deux attributs dans le shader
	VertexColorIndex = glGetAttribLocation(program, "VertexColor");
	VertexPositionIndex = glGetAttribLocation(program, "VertexPosition");
	
	checkError("initShader");
}

/** Initialisation des tats OpenGL et matrices */
void initRendering(void)
{
	glClearColor(1.0, 1.0, 1.0, 1.0);
    glClearStencil(0);
    glClearDepth(1.0f);

	// Activation du Back face culling
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);

	// Cration des matrices de transformation.
	projectionMatrix = perspective(60.0f, float(width) / float(height), 0.1f, 500.0f);
	
	modelWorldMatrix = translate(0.0f, 0.0f, -5.0f);
	modelWorldMatrix = rotate(modelWorldMatrix,40.0f,1.0f,1.0f,0.0f);

	worldViewMatrix = mat4(1.0);
	
	checkError("initRendering");
}

/** Cration d'un Vertex Array Object */
void createVAO()
{
	// Cration du VAO
	glGenVertexArrays(1, &vertexArrayObject);

	// On "bind" le VAO
	glBindVertexArray(vertexArrayObject);
	
	// On appelle toutes les fonctions qui modifient l'tat des tableaux de sommets
	glEnableVertexAttribArray(VertexPositionIndex);
	glEnableVertexAttribArray(VertexColorIndex);
	
	// On active le buffer des vertices
	glBindBuffer(GL_ARRAY_BUFFER, cubeBuffers[0]);
	
	// On envoie les donnes
	glVertexAttribPointer(VertexColorIndex, numColorComponents, GL_FLOAT, GL_FALSE, 
		stride, bufferObjectPtr(0));
	glVertexAttribPointer(VertexPositionIndex, numVertexComponents, GL_FLOAT, GL_FALSE, 
		stride, bufferObjectPtr(sizeof(GLfloat)*numVertexComponents));
			
	// On active le buffer des indices
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, cubeBuffers[1]);

	// On unbind le VAO, tout ceci est enregistr
	glBindVertexArray(0);

	checkError("createVAO");
}


/** Quad de la taille de l'cran pour la dtection de silhouettes */
void initQuad(void)
{
	quadProgram = createShaderProgram(std::string("Shaders/quad.vp"),std::string("Shaders/quad.fp"), 0);
	ColorTexIndex = glGetUniformLocation(quadProgram, "colorMap");
	DepthTexIndex = glGetUniformLocation(quadProgram, "depthMap");
	ThresholdIndex = glGetUniformLocation(quadProgram, "threshold");
	WidthIndex = glGetUniformLocation(quadProgram, "width");
	HeightIndex = glGetUniformLocation(quadProgram, "height");

	glGenBuffers(2, quadBuffer);
	
	glBindBuffer(GL_ARRAY_BUFFER, quadBuffer[0]);
	glBufferData(GL_ARRAY_BUFFER, sizeof(quadArray), quadArray, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, quadBuffer[1]);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(quadIndiceArray), quadIndiceArray, GL_STATIC_DRAW);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	glGenVertexArrays(1, &quadVAO);
	glBindVertexArray(quadVAO);

	GLuint posID=glGetAttribLocation( quadProgram, "VertexPosition" );
	glEnableVertexAttribArray(posID);

	GLuint uvID=glGetAttribLocation( quadProgram, "VertexUV" );
	if(uvID!=-1)
		glEnableVertexAttribArray(uvID);

	glBindBuffer(GL_ARRAY_BUFFER, quadBuffer[0]);
	
	glVertexAttribPointer(posID, 2.0f, GL_FLOAT, GL_FALSE, 
		sizeof(GLfloat)*4.0f, bufferObjectPtr(sizeof(GLfloat)*2.0));
	if(uvID!=-1)
	glVertexAttribPointer(uvID, 2.0f, GL_FLOAT, GL_FALSE, 
		sizeof(GLfloat)*4.0f, bufferObjectPtr(0));

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, quadBuffer[1]);

	glBindVertexArray(0);

	checkError ("initQuad");
}

void initCube(void) 
{
	initBuffer();
	
	if (glewIsSupported("GL_ARB_vertex_array_object")){
		createVAO();
	}else{
		fprintf(stderr,"VAO not supported");
		exit(EXIT_FAILURE);
	}
}

void initMaillage(void) 
{
	// ...dans Helpers.h
	initModel("Modeles/Lemming.3ds");	

	initMaillageAttrib(program);

	modelWorldMatrix = translate(-0.5f, -1.0f, -4.0f);
	modelWorldMatrix = rotate(modelWorldMatrix,40.0f,1.0f,1.0f,0.0f);
}

void initSphere(void) 
{
	_sphere.initAttrib(program);
	
	// Utilisation d'une texture de couleur
	initTexture("Textures/earth1.tga", &id_tex);
	ColorTexIndex = glGetUniformLocation(program, "Tex2D");

	// Utilisation d'une texture de bump
	initTexture("Textures/earth4.tga", &id_bump_tex);
	DepthTexIndex = glGetUniformLocation(program, "TexBump");

	modelWorldMatrix = mat4(1.0);
	modelWorldMatrix = rotate(modelWorldMatrix,200.0f,0.0f,1.0f,0.0f);

	lightPos = vec3(0.0f,0.0f,1.0f);
}

/** Diffrentes mthodes envoyant la gomtrie  chaque frame */
void drawQuad(void)
{
	glBindVertexArray(quadVAO);
	
	glDrawElements(GL_QUADS, 4, GL_UNSIGNED_INT, 0);

	glBindVertexArray(0);

	checkError("drawQuad");
}

void drawMaillage(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDepthMask(true);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);

	// On commence par activer le shader
	glUseProgram (program);

	/* worldViewMatrix = translate(0.0f,0.0f,-MyView.Distance);
	worldViewMatrix = rotate(worldViewMatrix, MyView.AngleX, 1.0f, 0.0f, 0.0f);
	worldViewMatrix = rotate(worldViewMatrix, MyView.AngleY, 0.0f, 1.0f, 0.0f);*/

	// On envoit les variables uniformes
	glUniformMatrix4fv(ModelViewMatrixIndex, 1, GL_FALSE, value_ptr(worldViewMatrix * modelWorldMatrix));
	glUniformMatrix4fv(ProjectionMatrixIndex, 1, GL_FALSE, value_ptr(projectionMatrix));	

	vec4 lightPosition = inverse(modelWorldMatrix)*vec4(2.0f,-2.0f,-2.0f,1.0f);
	/*vec4 lightPosition = rotate(Light.AngleY, vec3(0.0f, 1.0f, 0.0f))*vec4(lightPos,1.0f);
	lightPosition = rotate(Light.AngleX, vec3(1.0f, 0.0f, 0.0f))*lightPosition;
	lightPosition = translate(vec3(0.0f, 0.0f, -Light.Distance))*lightPosition;*/
	glUniform3f(LightPosIndex, lightPosition.x, lightPosition.y, lightPosition.z);

	// Rendu de la gomtrie
	glDrawElements(GL_TRIANGLES, getModelNumIndices(0), GL_UNSIGNED_SHORT, getModelIndices(0));

	// On dsactive le shader
	glUseProgram(0);

	checkError("drawMaillage");
}

void drawSphere(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDepthMask(true);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);

	// On commence par activer le shader
	glUseProgram (program);

	// On envoit les variables uniformes

	worldViewMatrix = translate(0.0f,0.0f,-MyView.Distance);
	worldViewMatrix = rotate(worldViewMatrix, MyView.AngleX, 1.0f, 0.0f, 0.0f);
	worldViewMatrix = rotate(worldViewMatrix, MyView.AngleY, 0.0f, 1.0f, 0.0f);

	glUniformMatrix4fv(ModelViewMatrixIndex, 1, GL_FALSE, value_ptr(worldViewMatrix * modelWorldMatrix));
	glUniformMatrix4fv(ProjectionMatrixIndex, 1, GL_FALSE, value_ptr(projectionMatrix));	

	vec4 lightPosition = rotate(Light.AngleY, vec3(0.0f, 1.0f, 0.0f))*vec4(lightPos,1.0f);
	lightPosition = rotate(Light.AngleX, vec3(1.0f, 0.0f, 0.0f))*lightPosition;
	lightPosition = translate(vec3(0.0f, 0.0f, -Light.Distance))*lightPosition;
	glUniform3f(LightPosIndex, lightPosition.x, lightPosition.y, lightPosition.z);

	glActiveTexture( GL_TEXTURE0 );
	glBindTexture(GL_TEXTURE_2D, id_tex);

	glUniform1i(ColorTexIndex,0);
	
	glActiveTexture( GL_TEXTURE1 );
	glBindTexture(GL_TEXTURE_2D, id_bump_tex);

	glUniform1i(DepthTexIndex,1);

	// Rendu de la gomtrie
	glDrawElements(GL_TRIANGLES, _sphere.getModelNumIndices(), GL_UNSIGNED_SHORT, _sphere.getModelIndices());

	// On dsactive le shader
	glUseProgram(0);

	checkError("drawSphere");
}

void drawCube(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDepthMask(true);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);

	// On commence par activer le shader
	glUseProgram (program);

	// On envoit les variables uniformes
	glUniformMatrix4fv(ModelViewMatrixIndex, 1, GL_FALSE, value_ptr(worldViewMatrix*modelWorldMatrix));
	glUniformMatrix4fv(ProjectionMatrixIndex, 1, GL_FALSE, value_ptr(projectionMatrix));
	glUniform3f(LightPosIndex, 1.0f, -2.0f, -2.0f);

	// On attache le VAO
	glBindVertexArray(vertexArrayObject);

	// Rendu de la gomtrie
	glDrawElements(GL_TRIANGLES, 36, GL_UNSIGNED_INT, 0);

	// On dtache le VAO
	glBindVertexArray(0);

	// On dsactive le shader
	glUseProgram(0);

	checkError("drawCube");
}

void defferedShading(void)
{
	// On active le framebuffer object
	glBindFramebuffer(GL_FRAMEBUFFER, fbo);
	// On choisi le buffer dans lequel crire la couleur
	glDrawBuffer(GL_COLOR_ATTACHMENT0);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDepthMask(true);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);

	// On commence par activer le shader
	glUseProgram (program);

	// On envoit les variables uniformes
	glUniform3f(LightPosIndex, lightPos.x, lightPos.y, lightPos.z);
	glUniformMatrix4fv(ModelViewMatrixIndex, 1, GL_FALSE, value_ptr(worldViewMatrix * modelWorldMatrix));
	glUniformMatrix4fv(ProjectionMatrixIndex, 1, GL_FALSE, value_ptr(projectionMatrix));	

	// On dessine la gomtrie
	glDrawElements(GL_TRIANGLES, getModelNumIndices(0), GL_UNSIGNED_SHORT, getModelIndices(0));

	glBindFramebuffer(GL_FRAMEBUFFER, 0);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);

	//// On commence par activer le shader du quad
	glUseProgram (quadProgram);

	glActiveTexture( GL_TEXTURE0 );
	glBindTexture(GL_TEXTURE_2D, colorMapTex);

	glActiveTexture( GL_TEXTURE1 );
	glBindTexture(GL_TEXTURE_2D, depthMapTex);

	glUniform1i(ColorTexIndex, 0);
	glUniform1i(DepthTexIndex, 1);
	glUniform1f(ThresholdIndex, 0.005);
	glUniform1f(WidthIndex, width);
	glUniform1f(HeightIndex, height);

	// On dessine le quad
	drawQuad();

	glActiveTexture( GL_TEXTURE0 );
	glDisable(GL_TEXTURE_2D);
	glActiveTexture( GL_TEXTURE1 );
	glDisable(GL_TEXTURE_2D);

	// On dsactive le shader
	glUseProgram(0);

}

/** Callback affichage */
void display(void)
{
	switch (shading) {
		case 0:
			drawCube();
			break;
		case 1:
			drawMaillage();
			break;
		case 2:
			drawSphere();
			break;
		case 3:
			defferedShading();
			break;
	}
	
	//On swap le front et le back buffer -> mise  jour de l'affichage
	glutSwapBuffers();

	checkError("display");
}

/** Callback cran redimensionn */
void reshape(int w, int h)
{
	// Pour viter les divisions par zro
	if(h == 0)
		h = 1;

	float ratio = float(w) / float(h);
	projectionMatrix = perspective(60.0f, ratio, 0.1f, 500.0f);
	
	glViewport(0, 0,(GLsizei) w,(GLsizei) h);

	width=w;
	height=h;

	checkError("reshape");
}

/** Callback clavier */
void keyboard(unsigned char key, int x, int y)
{
	switch(key) {
	case 27:
		exit(EXIT_SUCCESS);  // Touche ESCAPE
		break;
	case 'r':
		initShader();
	default:
		break;
	}
}

/** Callback pour les touches spciales */
void arrow_keys ( int a_keys, int x, int y )
{
  switch ( a_keys ) {
    case GLUT_KEY_F1:
    	glutFullScreen();
      break;
    case GLUT_KEY_F2:
    	glutReshapeWindow(500,500);
		glutPositionWindow(100,100);
      break;
    default:
      break;
  }
}

/** Callback pour les clics de la souris */
void mouse_click(int button, int state, int x, int y){
	int Modifiers;
	switch( button ) 
	{
		case GLUT_LEFT_BUTTON:
			MyMouse.Left = ( state == GLUT_DOWN ); break;
		case GLUT_MIDDLE_BUTTON:
			MyMouse.Middle = ( state == GLUT_DOWN ); break;
		case GLUT_RIGHT_BUTTON:
			MyMouse.Right = ( state == GLUT_DOWN ); 
			break;
	}

	Modifiers = glutGetModifiers();
	MyMouse.Shift = ( Modifiers & GLUT_ACTIVE_SHIFT ) ?1:0;
	MyMouse.Ctrl  = ( Modifiers & GLUT_ACTIVE_CTRL )  ?1:0;
	MyMouse.Alt   = ( Modifiers & GLUT_ACTIVE_ALT )   ?1:0;

	MyMouse.State = MOUSE_SHIFT*MyMouse.Shift | MOUSE_CTRL*MyMouse.Ctrl | MOUSE_ALT*MyMouse.Alt |
			MOUSE_LEFT*MyMouse.Left | MOUSE_MIDDLE*MyMouse.Middle | MOUSE_RIGHT*MyMouse.Right;


	MyMouse.X0 = MyMouse.X; MyMouse.X = x /(double)width; 		
	MyMouse.Dx = MyMouse.X0 - MyMouse.X;
	MyMouse.Y0 = MyMouse.Y; MyMouse.Y = (height-y)/(double)height; 
	MyMouse.Dy = MyMouse.Y0 - MyMouse.Y;

	switch( MyMouse.State ) 
	{
		case 0: 
			break;
		case MOUSE_LEFT:   
			break;
		case MOUSE_MIDDLE:
			break;
		case MOUSE_RIGHT:
			break;
		case MOUSE_SHIFT|MOUSE_LEFT:
			break;
		case MOUSE_SHIFT|MOUSE_MIDDLE:
			break;
		case MOUSE_SHIFT|MOUSE_RIGHT:
			break;
		case MOUSE_CTRL|MOUSE_LEFT:
			break;
		case MOUSE_CTRL|MOUSE_MIDDLE:
			break;
		case MOUSE_CTRL|MOUSE_RIGHT:
			break;
	}
	glutPostRedisplay();
}

/** Callback pour les mouvements passifs de la souris */
void mouse_motion(int x, int y){
	MyMouse.X0 = MyMouse.X;
	MyMouse.Y0 = MyMouse.Y;
	MyMouse.X =          x /(double)width; 
	MyMouse.Y = (height-y)/(double)height;
	MyMouse.Dx = MyMouse.X-MyMouse.X0;
	MyMouse.Dy = MyMouse.Y-MyMouse.Y0;
}

/** Callback pour les mouvement actifs de la souris */
void mouse_drag(int x, int y){
	MyMouse.X0 = MyMouse.X;
	MyMouse.Y0 = MyMouse.Y;
	MyMouse.X =  x /(double)width; 
	MyMouse.Y = (height-y)/(double)height;
	MyMouse.Dx = MyMouse.X-MyMouse.X0;
	MyMouse.Dy = MyMouse.Y-MyMouse.Y0;

	switch( MyMouse.State )
	{
		case 0: 
		break;
		case MOUSE_LEFT: 
			MyView.AngleY += MyMouse.Dx*50;
			MyView.AngleX -= MyMouse.Dy*50;
		break;
		case MOUSE_MIDDLE:
			MyView.Distance -= MyMouse.Dy * MyView.Distance;
		break;
		case MOUSE_RIGHT:
			Light.AngleY += MyMouse.Dx*50;
			Light.AngleX += MyMouse.Dy*50;
			break;
		case MOUSE_SHIFT|MOUSE_LEFT:
			break;
		case MOUSE_SHIFT|MOUSE_MIDDLE:
			break;
		case MOUSE_SHIFT|MOUSE_RIGHT:
			break;
		case MOUSE_CTRL|MOUSE_LEFT:
			break;
		case MOUSE_CTRL|MOUSE_MIDDLE:
			Light.Distance -= MyMouse.Dy * Light.Distance;
			break;
		case MOUSE_CTRL|MOUSE_RIGHT:
			break;
	}

	glutPostRedisplay();
}

/** Initialisatinon de GLUT */
int initGLUT(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH | GLUT_STENCIL);
	glutInitContextVersion (3, 0);
	glutInitWindowSize(width, height); 
	glutInitWindowPosition(100, 100);
	int handle = glutCreateWindow("TP OpenGL");

	/** Spcification des callbacks pour l'affichage, le redimensionnement et le clavier */
	glutDisplayFunc(display); 
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(arrow_keys);
	glutMotionFunc(mouse_drag);
	glutPassiveMotionFunc(mouse_motion);
	glutMouseFunc(mouse_click);
	glutIdleFunc(display);

	dumpInfo();

	checkError("initGLUT");
	return handle;
}

/**  Initialisatinon de GLEW */
void initGLEW()
{
	GLenum error = glewInit();

	if(GLEW_OK != error)
	{
		// Erreur lors du chargement de GLEW
		fprintf(stderr, "Error: %s\n", glewGetErrorString(error));
		exit(EXIT_FAILURE);
	}

	checkError("initGLEW");
}

/** Libration de la mmoire */
void clean ()
{
	glDeleteProgram(program);
	glDeleteBuffers(2, cubeBuffers);
	glDeleteVertexArrays(1, &vertexArrayObject);
}

int main(int argc, char** argv)
{
	//Initialisatinon de GLUT
	initGLUT(argc,argv);
	
	//Initialisatinon de GLEW
	initGLEW();

	//Pour choisir le rendu
	shading=0;

	//Paramtres initiaux de la camra et de la lampe
	MyView.Distance = 5;
	MyView.AngleY = 0;
	MyView.AngleX = 0;
	Light.Distance = 5;
	Light.AngleY = 0;
	Light.AngleX = 0;

	initShader();
	initRendering();

	switch(shading){
		case 0:
			initCube();
			break;
		case 1:
			initMaillage();
			break;
		case 2 :
			initSphere();
			break;
		case 3:
			initFBO();
			initMaillage();
			initQuad();
			break;
	}

	/** Boucle d'affichage principale */
	glutMainLoop();

	clean();

	return EXIT_SUCCESS;
}
